/*
 * HSV2RGB.c
 *
 * Created: 27.12.2016 11:08:25
 *  Author: Thomas
 */ 

#include <avr/io.h>

#include "HSV2RGB.h"


void HSV2RGB(pHSV_T pHSV, pRGB_T pRGB)
{
	uint8_t H = pHSV->H;
	uint8_t S = pHSV->S;
	uint8_t V = pHSV->V;

	if(S == 0)
	{
		pRGB->Red = pRGB->Green = pRGB->Blue = V;
	}
	else
	{
		uint16_t tmpH = H * 6;
		uint8_t i = tmpH >> 8;
		uint8_t f = (uint8_t)tmpH;

		
		// p = (V * (255 - S)) >> 8 (needed in each region)
		uint8_t p = 255 - S;
				p = (V * p) >> 8;
		
		#ifdef HSV2RGB_OPTIMIZED_SIZE
			// q = (V * (255 - ((S * f) >> 8))) >> 8
			uint8_t q = (S * f) >> 8;
					q = 255 - q;
					q = (V * q) >> 8;

			// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
			uint8_t t = 255 - f;
					t = (S * t) >> 8;
					t = 255 - t;
					t = (V * t) >> 8;
		#endif
		
		
		if(i == 0)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = V; g = t; b = p;
			pRGB->Red = V;
			pRGB->Green = t;
			pRGB->Blue = p;
		}
		else if (i == 1)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = q; g = V; b = p;
			pRGB->Red = q;
			pRGB->Green = V;
			pRGB->Blue = p;
		}
		else if (i == 2)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
				t = (S * t) >> 8;
				t = 255 - t;
				t = (V * t) >> 8;
			#endif

			// r = p; g = V; b = t;
			pRGB->Red = p;
			pRGB->Green = V;
			pRGB->Blue = t;
		}
		else if (i == 3)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = p; g = q; b = V;
			pRGB->Red = p;
			pRGB->Green = q;
			pRGB->Blue = V;
		}
		else if (i == 4)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			//r = t; g = p; b = V;
			pRGB->Red = t;
			pRGB->Green = p;
			pRGB->Blue = V;
		}
		else
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = V; g = p; b = q;
			pRGB->Red = V;
			pRGB->Green = p;
			pRGB->Blue = q;
		}
	}
}


void HSV2GRB(pHSV_T pHSV, pGRB_T pGRB)
{
	uint8_t H = pHSV->H;
	uint8_t S = pHSV->S;
	uint8_t V = pHSV->V;

	if(S == 0)
	{
		pGRB->Red = pGRB->Green = pGRB->Blue = V;
	}
	else
	{
		uint16_t tmpH = H * 6;
		uint8_t i = tmpH >> 8;
		uint8_t f = (uint8_t)tmpH;

		
		// p = (V * (255 - S)) >> 8 (needed in each region)
		uint8_t p = 255 - S;
				p = (V * p) >> 8;
		
		#ifdef HSV2RGB_OPTIMIZED_SIZE
			// q = (V * (255 - ((S * f) >> 8))) >> 8
			uint8_t q = (S * f) >> 8;
					q = 255 - q;
					q = (V * q) >> 8;

			// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
			uint8_t t = 255 - f;
					t = (S * t) >> 8;
					t = 255 - t;
					t = (V * t) >> 8;
		#endif
		
		
		if(i == 0)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = V; g = t; b = p;
			pGRB->Red = V;
			pGRB->Green = t;
			pGRB->Blue = p;
		}
		else if (i == 1)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = q; g = V; b = p;
			pGRB->Red = q;
			pGRB->Green = V;
			pGRB->Blue = p;
		}
		else if (i == 2)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = p; g = V; b = t;
			pGRB->Red = p;
			pGRB->Green = V;
			pGRB->Blue = t;
		}
		else if (i == 3)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = p; g = q; b = V;
			pGRB->Red = p;
			pGRB->Green = q;
			pGRB->Blue = V;
		}
		else if (i == 4)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			//r = t; g = p; b = V;
			pGRB->Red = t;
			pGRB->Green = p;
			pGRB->Blue = V;
		}
		else
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = V; g = p; b = q;
			pGRB->Red = V;
			pGRB->Green = p;
			pGRB->Blue = q;
		}
	}
}


void HSV2BGR(pHSV_T pHSV, pBGR_T pBGR)
{
	uint8_t H = pHSV->H;
	uint8_t S = pHSV->S;
	uint8_t V = pHSV->V;

	if(S == 0)
	{
		pBGR->Red = pBGR->Green = pBGR->Blue = V;
	}
	else
	{
		uint16_t tmpH = H * 6;
		uint8_t i = tmpH >> 8;
		uint8_t f = (uint8_t)tmpH;

		
		// p = (V * (255 - S)) >> 8 (needed in each region)
		uint8_t p = 255 - S;
				p = (V * p) >> 8;
		
		#ifdef HSV2RGB_OPTIMIZED_SIZE
			// q = (V * (255 - ((S * f) >> 8))) >> 8
			uint8_t q = (S * f) >> 8;
					q = 255 - q;
					q = (V * q) >> 8;

			// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
			uint8_t t = 255 - f;
					t = (S * t) >> 8;
					t = 255 - t;
					t = (V * t) >> 8;
		#endif
		
		
		if(i == 0)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = V; g = t; b = p;
			pBGR->Red = V;
			pBGR->Green = t;
			pBGR->Blue = p;
		}
		else if (i == 1)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = q; g = V; b = p;
			pBGR->Red = q;
			pBGR->Green = V;
			pBGR->Blue = p;
		}
		else if (i == 2)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = p; g = V; b = t;
			pBGR->Red = p;
			pBGR->Green = V;
			pBGR->Blue = t;
		}
		else if (i == 3)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = p; g = q; b = V;
			pBGR->Red = p;
			pBGR->Green = q;
			pBGR->Blue = V;
		}
		else if (i == 4)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			//r = t; g = p; b = V;
			pBGR->Red = t;
			pBGR->Green = p;
			pBGR->Blue = V;
		}
		else
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = V; g = p; b = q;
			pBGR->Red = V;
			pBGR->Green = p;
			pBGR->Blue = q;
		}
	}
}


void HSV2APA102cGBGR(pHSV_T pHSV, pAPA102GBGR_T pAPA102GBGR)
{
	uint8_t H = pHSV->H;
	uint8_t S = pHSV->S;
	uint8_t V = pHSV->V;

	if(S == 0)
	{
		pAPA102GBGR->Red = pAPA102GBGR->Green = pAPA102GBGR->Blue = V;
	}
	else
	{
		uint16_t tmpH = H * 6;
		uint8_t i = tmpH >> 8;
		uint8_t f = (uint8_t)tmpH;

		
		// p = (V * (255 - S)) >> 8 (needed in each region)
		uint8_t p = 255 - S;
				p = (V * p) >> 8;
		
		#ifdef HSV2RGB_OPTIMIZED_SIZE
			// q = (V * (255 - ((S * f) >> 8))) >> 8
			uint8_t q = (S * f) >> 8;
					q = 255 - q;
					q = (V * q) >> 8;

			// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
			uint8_t t = 255 - f;
					t = (S * t) >> 8;
					t = 255 - t;
					t = (V * t) >> 8;
		#endif
		
		
		if(i == 0)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = V; g = t; b = p;
			pAPA102GBGR->Red = V;
			pAPA102GBGR->Green = t;
			pAPA102GBGR->Blue = p;
		}
		else if (i == 1)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = q; g = V; b = p;
			pAPA102GBGR->Red = q;
			pAPA102GBGR->Green = V;
			pAPA102GBGR->Blue = p;
		}
		else if (i == 2)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			// r = p; g = V; b = t;
			pAPA102GBGR->Red = p;
			pAPA102GBGR->Green = V;
			pAPA102GBGR->Blue = t;
		}
		else if (i == 3)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = p; g = q; b = V;
			pAPA102GBGR->Red = p;
			pAPA102GBGR->Green = q;
			pAPA102GBGR->Blue = V;
		}
		else if (i == 4)
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
				uint8_t t = 255 - f;
						t = (S * t) >> 8;
						t = 255 - t;
						t = (V * t) >> 8;
			#endif

			//r = t; g = p; b = V;
			pAPA102GBGR->Red = t;
			pAPA102GBGR->Green = p;
			pAPA102GBGR->Blue = V;
		}
		else
		{
			#ifndef HSV2RGB_OPTIMIZED_SIZE
				// q = (V * (255 - ((S * f) >> 8))) >> 8
				uint8_t q = (S * f) >> 8;
						q = 255 - q;
						q = (V * q) >> 8;
			#endif

			// r = V; g = p; b = q;
			pAPA102GBGR->Red = V;
			pAPA102GBGR->Green = p;
			pAPA102GBGR->Blue = q;
		}
	}
}
			 

